@@ -56,10 +56,8 @@ pub enum Error {
     InvalidData(&'static str),
     /// Parse error caused by limited parser support rather than invalid data.
     Unsupported(&'static str),
-    /// Reflect `byteorder::Error::UnexpectedEOF` for short data.
+    /// Reflect `std::io::ErrorKind::UnexpectedEof` for short data.
     UnexpectedEOF,
-    /// Caught panic! or assert! meaning the parser couldn't recover.
-    AssertCaught,
     /// Propagate underlying errors from `std::io`.
     Io(std::io::Error),
     /// read_mp4 terminated without detecting a moov box.
@@ -68,15 +66,9 @@ pub enum Error {
 
 impl From<std::io::Error> for Error {
     fn from(err: std::io::Error) -> Error {
-        Error::Io(err)
-    }
-}
-
-impl From<byteorder::Error> for Error {
-    fn from(err: byteorder::Error) -> Error {
-        match err {
-            byteorder::Error::UnexpectedEOF => Error::UnexpectedEOF,
-            byteorder::Error::Io(e) => Error::Io(e),
+        match err.kind() {
+            std::io::ErrorKind::UnexpectedEof => Error::UnexpectedEOF,
+            _ => Error::Io(err),
         }
     }
 }
@@ -1032,6 +1024,49 @@ fn read_dops<T: Read>(src: &mut BMFFBox<T>) -> Result<OpusSpecificBox> {
     })
 }
 
+/// Re-serialize the Opus codec-specific config data as an OpusHead packet.
+///
+/// Some decoders expect the initialization data in the format used by the
+/// Ogg and WebM encapsulations. To support this we prepend the 'OpusHead'
+/// tag and byte-swap the data from big- to little-endian relative to the
+/// dOps box.
+fn serialize_opus_header<W: byteorder::WriteBytesExt + std::io::Write>(opus: &OpusSpecificBox, dst: &mut W) -> Result<()> {
+    match dst.write(b"OpusHead") {
+        Err(e) => return Err(Error::from(e)),
+        Ok(bytes) => {
+            if bytes != 8 {
+                return Err(Error::InvalidData("Couldn't write OpusHead tag."));
+            }
+        }
+    }
+    // In mp4 encapsulation, the version field is 0, but in ogg
+    // it is 1. While decoders generally accept zero as well, write
+    // out the version of the header we're supporting rather than
+    // whatever we parsed out of mp4.
+    try!(dst.write_u8(1));
+    try!(dst.write_u8(opus.output_channel_count));
+    try!(dst.write_u16::<byteorder::LittleEndian>(opus.pre_skip));
+    try!(dst.write_u32::<byteorder::LittleEndian>(opus.input_sample_rate));
+    try!(dst.write_i16::<byteorder::LittleEndian>(opus.output_gain));
+    try!(dst.write_u8(opus.channel_mapping_family));
+    match opus.channel_mapping_table {
+        None => {}
+        Some(ref table) => {
+            try!(dst.write_u8(table.stream_count));
+            try!(dst.write_u8(table.coupled_count));
+            match dst.write(&table.channel_mapping) {
+                Err(e) => return Err(Error::from(e)),
+                Ok(bytes) => {
+                    if bytes != table.channel_mapping.len() {
+                        return Err(Error::InvalidData("Couldn't write channel mapping table data."));
+                    }
+                }
+            }
+        }
+    };
+    Ok(())
+}
+
 /// Parse a hdlr box.
 fn read_hdlr<T: Read>(src: &mut BMFFBox<T>) -> Result<HandlerBox> {
     let (_, _) = try!(read_fullbox_extra(src));
@@ -1328,26 +1363,26 @@ fn track_time_to_ms(time: TrackScaledTime, scale: TrackTimeScale) -> u64 {
     time.0 * 1000000 / scale.0
 }
 
-fn be_i16<T: ReadBytesExt>(src: &mut T) -> byteorder::Result<i16> {
-    src.read_i16::<byteorder::BigEndian>()
+fn be_i16<T: ReadBytesExt>(src: &mut T) -> Result<i16> {
+    src.read_i16::<byteorder::BigEndian>().map_err(From::from)
 }
 
-fn be_i32<T: ReadBytesExt>(src: &mut T) -> byteorder::Result<i32> {
-    src.read_i32::<byteorder::BigEndian>()
+fn be_i32<T: ReadBytesExt>(src: &mut T) -> Result<i32> {
+    src.read_i32::<byteorder::BigEndian>().map_err(From::from)
 }
 
-fn be_i64<T: ReadBytesExt>(src: &mut T) -> byteorder::Result<i64> {
-    src.read_i64::<byteorder::BigEndian>()
+fn be_i64<T: ReadBytesExt>(src: &mut T) -> Result<i64> {
+    src.read_i64::<byteorder::BigEndian>().map_err(From::from)
 }
 
-fn be_u16<T: ReadBytesExt>(src: &mut T) -> byteorder::Result<u16> {
-    src.read_u16::<byteorder::BigEndian>()
+fn be_u16<T: ReadBytesExt>(src: &mut T) -> Result<u16> {
+    src.read_u16::<byteorder::BigEndian>().map_err(From::from)
 }
 
-fn be_u32<T: ReadBytesExt>(src: &mut T) -> byteorder::Result<u32> {
-    src.read_u32::<byteorder::BigEndian>()
+fn be_u32<T: ReadBytesExt>(src: &mut T) -> Result<u32> {
+    src.read_u32::<byteorder::BigEndian>().map_err(From::from)
 }
 
-fn be_u64<T: ReadBytesExt>(src: &mut T) -> byteorder::Result<u64> {
-    src.read_u64::<byteorder::BigEndian>()
+fn be_u64<T: ReadBytesExt>(src: &mut T) -> Result<u64> {
+    src.read_u64::<byteorder::BigEndian>().map_err(From::from)
 }
